@@ -13,6 +13,7 @@
 __docformat__ = 'restructuredtext'
 
 from collections import OrderedDict
+from itertools import chain
 import logging
 import re
 
@@ -498,7 +499,7 @@ def _push(dspath, content, target, data, force, jobs, res_kwargs, pbars,
     depvar = 'remote.{}.datalad-publish-depends'.format(target)
     # list of remotes that are publication dependencies for the
     # target remote
-    publish_depends = ensure_list(ds.config.get(depvar, []))
+    publish_depends = ensure_list(ds.config.get(depvar, [], get_all=True))
     if publish_depends:
         lgr.debug("Discovered publication dependencies for '%s': %s'",
                   target, publish_depends)
