@@ -10,6 +10,8 @@
 Thread based subprocess execution with stdout and stderr passed to protocol objects
 """
 
+from __future__ import annotations
+
 import enum
 import logging
 import subprocess
@@ -21,14 +23,11 @@ from queue import (
     Empty,
     Queue,
 )
+from subprocess import Popen
 from typing import (
     Any,
-    Dict,
     IO,
-    List,
     Optional,
-    Type,
-    Union,
 )
 
 from datalad.utils import on_windows
@@ -46,7 +45,6 @@ from .runnerthreads import (
     WriteThread,
 )
 
-
 lgr = logging.getLogger("datalad.runner.nonasyncrunner")
 
 STDIN_FILENO = 0
@@ -54,6 +52,17 @@ STDOUT_FILENO = 1
 STDERR_FILENO = 2
 
 
+# A helper to type-safe retrieval of a Popen-fileno, if data exchange was
+# requested.
+def _get_fileno(active: bool,
+                popen_std_x: Optional[IO]
+                ) -> Optional[int]:
+    if active:
+        assert popen_std_x is not None
+        return popen_std_x.fileno()
+    return None
+
+
 class _ResultGenerator(Generator):
     """
     Generator returned by run_command if the protocol class
@@ -66,7 +75,7 @@ class _ResultGenerator(Generator):
         waiting_for_process = 3
 
     def __init__(self,
-                 runner: "ThreadedRunner",
+                 runner: ThreadedRunner,
                  result_queue: deque
                  ):
 
@@ -137,10 +146,10 @@ class ThreadedRunner:
     timeout_resolution = 0.2
 
     def __init__(self,
-                 cmd: Union[str, List],
-                 protocol_class: Type[WitlessProtocol],
+                 cmd: str | list,
+                 protocol_class: type[WitlessProtocol],
                  stdin: Any,
-                 protocol_kwargs: Optional[Dict] = None,
+                 protocol_kwargs: Optional[dict] = None,
                  timeout: Optional[float] = None,
                  exception_on_error: bool = True,
                  **popen_kwargs
@@ -215,38 +224,40 @@ class ThreadedRunner:
         self.exception_on_error = exception_on_error
         self.popen_kwargs = popen_kwargs
 
-        self.catch_stdout = self.protocol_class.proc_out is not None
-        self.catch_stderr = self.protocol_class.proc_err is not None
-        self.generator = self.protocol_class.generator is not None
+        self.catch_stdout = self.protocol_class.proc_out
+        self.catch_stderr = self.protocol_class.proc_err
 
-        self.write_stdin = False
-        self.stdin_queue = None
-        self.protocol = None
-        self.process = None
-        self.process_stdin_fileno = None
-        self.process_stdout_fileno = None
-        self.process_stderr_fileno = None
-        self.stderr_enqueueing_thread = None
-        self.stdout_enqueueing_thread = None
-        self.stdin_enqueueing_thread = None
-        self.process_waiting_thread = None
+        self.write_stdin: bool = False
+        self.stdin_queue: Optional[Queue] = None
+        self.process_stdin_fileno: Optional[int] = None
+        self.process_stdout_fileno: Optional[int] = None
+        self.process_stderr_fileno: Optional[int] = None
+        self.stderr_enqueueing_thread: Optional[ReadThread] = None
+        self.stdout_enqueueing_thread: Optional[ReadThread] = None
+        self.stdin_enqueueing_thread: Optional[WriteThread] = None
+        self.process_waiting_thread: Optional[WaitThread] = None
 
-        self.process_running = False
-        self.fileno_mapping = None
-        self.fileno_to_file = None
-        self.file_to_fileno = None
-        self.output_queue = Queue()
-        self.result = None
-        self.process_removed = False
-        self.return_code = None
-        self.generator = None
+        self.process_running: bool = False
+        self.output_queue: Queue = Queue()
+        self.process_removed: bool = False
+        self.generator: Optional[_ResultGenerator] = None
 
-        self.last_touched = dict()
-        self.active_file_numbers = set()
+        self.last_touched: dict[Optional[int], float] = dict()
+        self.active_file_numbers: set[Optional[int]] = set()
         self.stall_check_interval = 10
 
         self.initialization_lock = threading.Lock()
 
+        # Pure declarations
+        self.protocol: WitlessProtocol
+        self.process: Popen[Any]
+        self.fileno_mapping: dict[Optional[int], int]
+        self.fileno_to_file: dict[Optional[int], Optional[IO]]
+        self.file_to_fileno: dict[IO, int]
+        self.result: dict
+        self.return_code: int
+
+
     def _check_result(self):
         if self.exception_on_error is True:
             if self.return_code != 0:
@@ -258,12 +269,14 @@ class ThreadedRunner:
                         ("stderr", protocol.stderr_fileno))
                     if protocol.fd_infos[fileno][1] is not None
                 }
-                raise CommandError(cmd=self.cmd,
-                                   code=self.return_code,
-                                   stdout=decoded_output.get("stdout", None),
-                                   stderr=decoded_output.get("stderr", None))
+                raise CommandError(
+                    cmd=self.cmd,
+                    code=self.return_code,
+                    stdout=decoded_output.get("stdout", None),
+                    stderr=decoded_output.get("stderr", None)
+                )
 
-    def run(self) -> Union[Any, Generator]:
+    def run(self) -> dict | Generator:
         """
         Run the command as specified in __init__.
 
@@ -306,7 +319,7 @@ class ThreadedRunner:
         with self.initialization_lock:
             return self._locked_run()
 
-    def _locked_run(self) -> Union[Any, Generator]:
+    def _locked_run(self) -> dict | Generator:
         if self.generator is not None:
             raise RuntimeError("ThreadedRunner.run() was re-entered")
 
@@ -352,7 +365,7 @@ class ThreadedRunner:
         try:
             # The following command is generated internally by datalad
             # and trusted. Security check is therefore skipped.
-            self.process = subprocess.Popen(self.cmd, **kwargs)         # nosec
+            self.process = Popen(self.cmd, **kwargs)         # nosec
 
         except OSError as e:
             if not on_windows and "argument list too long" in str(e).lower():
@@ -369,9 +382,9 @@ class ThreadedRunner:
         self.process_running = True
         self.active_file_numbers.add(None)
 
-        self.process_stdin_fileno = self.process.stdin.fileno() if self.write_stdin else None
-        self.process_stdout_fileno = self.process.stdout.fileno() if self.catch_stdout else None
-        self.process_stderr_fileno = self.process.stderr.fileno() if self.catch_stderr else None
+        self.process_stdin_fileno = _get_fileno(self.write_stdin, self.process.stdin)
+        self.process_stdout_fileno = _get_fileno(self.catch_stdout, self.process.stdout)
+        self.process_stderr_fileno = _get_fileno(self.catch_stderr, self.process.stderr)
 
         # We pass process as transport-argument. It does not have the same
         # semantics as the asyncio-signature, but since it is only used in
@@ -385,12 +398,16 @@ class ThreadedRunner:
             self.process_stderr_fileno: STDERR_FILENO,
             self.process_stdin_fileno: STDIN_FILENO,
         }
+        if None in self.fileno_mapping:
+            self.fileno_mapping.pop(None)
 
         self.fileno_to_file = {
             self.process_stdout_fileno: self.process.stdout,
             self.process_stderr_fileno: self.process.stderr,
             self.process_stdin_fileno: self.process.stdin
         }
+        if None in self.fileno_to_file:
+            self.fileno_to_file.pop(None)
 
         self.file_to_fileno = {
             f: f.fileno()
@@ -411,6 +428,7 @@ class ThreadedRunner:
                 self.last_touched[self.process_stderr_fileno] = current_time
             self.active_file_numbers.add(self.process_stderr_fileno)
             self.last_touched[self.process_stderr_fileno] = current_time
+            assert self.process.stderr is not None
             self.stderr_enqueueing_thread = ReadThread(
                 identifier="STDERR: " + cmd_string[:20],
                 signal_queues=[self.output_queue],
@@ -424,6 +442,7 @@ class ThreadedRunner:
                 self.last_touched[self.process_stdout_fileno] = current_time
             self.active_file_numbers.add(self.process_stdout_fileno)
             self.last_touched[self.process_stdout_fileno] = current_time
+            assert self.process.stdout is not None
             self.stdout_enqueueing_thread = ReadThread(
                 identifier="STDOUT: " + cmd_string[:20],
                 signal_queues=[self.output_queue],
@@ -435,6 +454,8 @@ class ThreadedRunner:
         if self.write_stdin:
             # No timeouts for stdin
             self.active_file_numbers.add(self.process_stdin_fileno)
+            assert self.stdin_queue is not None
+            assert self.process.stdin is not None
             self.stdin_enqueueing_thread = WriteThread(
                 identifier="STDIN: " + cmd_string[:20],
                 user_info=self.process_stdin_fileno,
@@ -449,13 +470,16 @@ class ThreadedRunner:
             self.process)
         self.process_waiting_thread.start()
 
-        if issubclass(self.protocol_class, GeneratorMixIn):
-            self.generator = _ResultGenerator(self, self.protocol.result_queue)
+        if isinstance(self.protocol, GeneratorMixIn):
+            self.generator = _ResultGenerator(
+                self,
+                self.protocol.result_queue
+            )
             return self.generator
 
         return self.process_loop()
 
-    def process_loop(self) -> Any:
+    def process_loop(self) -> dict:
         # Process internal messages until no more active file descriptors
         # are present. This works because active file numbers are only
         # removed when an EOF is received in `self.process_queue`.
@@ -515,8 +539,8 @@ class ThreadedRunner:
             or not self.output_queue.empty())
 
     def is_stalled(self) -> bool:
-        # If all threads have exited and the queue is empty, we might have a
-        # stall condition.
+        # If all queue-filling threads have exited and the queue is empty, we
+        # might have a stall condition.
         live_threads = [
             thread.is_alive()
             for thread in (
@@ -656,13 +680,13 @@ class ThreadedRunner:
                 thread.request_exit()
 
 
-def run_command(cmd: Union[str, List],
-                protocol: Type[WitlessProtocol],
+def run_command(cmd: str | list,
+                protocol: type[WitlessProtocol],
                 stdin: Any,
-                protocol_kwargs: Optional[Dict] = None,
+                protocol_kwargs: Optional[dict] = None,
                 timeout: Optional[float] = None,
                 exception_on_error: bool = True,
-                **popen_kwargs) -> Union[Any, Generator]:
+                **popen_kwargs) -> dict | Generator:
     """
     Run a command in a subprocess
 
