@@ -190,7 +190,6 @@ def _check_ri(ri, cls, exact_str=True, localpath=None, **fields):
         eq_(ri_.localpath, opj(old_localpath, 'sub'))
 
 
-
 def test_url_base():
     # Basic checks
     assert_raises(ValueError, URL, "http://example.com", hostname='example.com')
@@ -219,6 +218,20 @@ def test_url_base():
         eq_(purl.as_str(), 'http://example.com/')
 
 
+@with_tempfile
+def test_pathri_guessing(filename=None):
+    # Complaining about ;param only at DEBUG level
+    # see https://github.com/datalad/datalad/issues/6872
+    with swallow_logs(new_level=logging.DEBUG) as cml:
+        # we don't "care" about params ATM so there is a warning if there are any
+        ri = RI(f"{filename};param")
+        assert isinstance(ri, PathRI)
+        if not on_windows:
+            # Does not happen on Windows since paths with \ instead of / do not
+            # look like possible URLs
+            assert_in('ParseResults contains params', cml.out)
+
+
 @known_failure_githubci_win
 def test_url_samples():
     _check_ri("http://example.com", URL, scheme='http', hostname="example.com")
